import {
	AbstractViewInterface,
	ApiInterface,
	CustomLayer,
	ImageInterface,
	PoiInterface,
	RestFilterInterface,
	SiteModelEntityInterface,
	TransitionEndEventInterface,
	TransitionStartEventInterface,
} from "@navvis/ivion";
import {escape} from "lodash";

export class PoiFilterMenu extends CustomLayer
{
	constructor(view: AbstractViewInterface,
	            protected ivion: ApiInterface)
	{
		super(view);

		const image = this.ivion.view.mainView.getImage();
		this.createPoiHtmlInformationPanel(image);
	}

	public onTransitionStart(event: TransitionStartEventInterface): void
	{
		this.resetHtml();
	}

	/**
	 * Reset the html.
	 */
	private resetHtml(): void
	{
		this.showHtml("");
	}

	public onTransitionEnd(event: TransitionEndEventInterface): void
	{
		this.createPoiHtmlInformationPanel(event.image);
	}

	/**
	 * Create the html panel that will hold information regarding POIs inside the current site
	 * entity.
	 * @param image The image that hold the information about the current position
	 */
	private createPoiHtmlInformationPanel(image: ImageInterface): void
	{
		if (!image)
		{
			return;
		}
		const filterBuilder = this.poisInCurrentSiteModelEntity(image.siteModelEntity);
		this.ivion.poi.repository.filter(filterBuilder)
			.then((pois: PoiInterface[]) => this.renderHtml(pois))
			.catch(() => {});
	}

	/**
	 * Create the html to display the provided pois in the page.
	 * If no pois are passed or the array is empty, do not render anything
	 * @param pois The pois to be displayed in html
	 */
	private renderHtml(pois: PoiInterface[]): void
	{
		if (!pois || pois.length <= 0)
		{
			return;
		}

		let poisHtml: string = "<h2>POIs in this site entity</h2>";
		for (const poi of pois)
		{
			poisHtml = poisHtml.concat(PoiFilterMenu.getPoiHtml(poi));
		}
		this.showHtml(poisHtml);
	}

	/**
	 * Create the filter needed to fetch POIs in the current site entity.
	 * The results will be limited to 5.
	 * @param currentSiteModelEntity The current site entity
	 */
	private poisInCurrentSiteModelEntity(currentSiteModelEntity: SiteModelEntityInterface)
		: RestFilterInterface
	{
		return this.ivion.poi.repository.createFilterBuilder()
			.siteModel([currentSiteModelEntity])
			.limit(5)
			.build();
	}

	/**
	 * Assign the provided html to the `pois-list` html element in the page.
	 * @param html The html to assign
	 */
	private showHtml(html: string): void
	{
		document.getElementById("pois-list").innerHTML = html;
	}

	/**
	 * Create the html to render a single provided poi.
	 * @param poi The poi to render in html
	 */
	private static getPoiHtml(poi: PoiInterface): string
	{
		return `<div id="poi-title">${escape(poi.title)}</div>
				<div id="poi-type">${escape(poi.poiType.name["en"])}<div>`;
	}
}
